--[[
Name: RC4-1.0
Revision: $Rev: 25931 $
Author: Sal Scotto (sal.scotto@gmail.com)
Website: http://www.wowace.com
Documentation: need to add wiki entry for it
SVN: $HeadURL: http://svn.wowace.com/wowace/trunk/RC4Lib/RC4-1.0/RC4-1.0.lua $
Description: Provides ciphersaber2 encryption routines
Dependencies: AceLibrary
]]

local vmajor,vminor = "RC4-1.0", "$Revision: 25931 $"

if not AceLibrary then error(vmajor .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(vmajor, vminor) then return end

-- this library work with Lua 5.0 and up, some better code can be done with Lua 5.1 but not really an issue right now
local cryptoLib = {}

-- Activate a new instance of the library
local function activate(self, oldLib, oldDeactivate)
	if oldDeactive then oldDeactivate(oldLib) end
end

-- generate iv, initialization vector
local function generate_iv()
	local outputkey = ""
	local iv = {0,0,0,0,0,0,0,0,0,0}
	math.randomseed(os.time())
	-- pop the first one off, bug on windows where first on inst random
	math.random()
	for i=0, 10,1 do
		local date = os.date()
		local offset = math.random(1,string.len(date))
		local oByte = string.byte(date,offset)
		local rs = math.random(255)
		iv[i] = bit.bxor(oByte,rs)
	end
	for x,byte in ipairs(iv) do
		outputkey = outputkey..string.char(byte)
	end
	return outputkey
end

-- perform the cipher function
local function ciphersaber2(key, text)
	-- CipherSaber 2 encryption 
	-- http://ciphersaber.gurus.com/
	local s = {}
	local i,j, klen, tlen,x, cs,csl
	klen = string.len(key)
	tlen = string.len(text)
	csl = 2
	-- KSA
	for i=0,256,1 do
		s[i] = i
	end
	j = 0
	-- ciphersaber 2 setup
	for cs = 0, csl, 1 do
		for i=0,256,1 do
			local kindex = bit.mod(i,klen)
			local kbyte = string.byte(key,kindex)
			if not kbyte then kbyte = 1 end
			j = bit.mod((j + s[i] + kbyte),256)
			local sb = s[i]
			s[i] = s[j]
			s[j] = sb
		end
	end
	-- PRGA
	local output = ""
	i = 0
	j = 0
	for x=1,tlen,1 do
		i = bit.mod((i + 1),255)
		j = bit.mod((j + s[i]), 255)
		local sb = s[i]
		s[i] = s[j]
		s[j] = sb
		-- working on this output byte
		local wbyte = string.byte(text,x)
		local mbyte = s[bit.mod((s[i] + s[j]),255)]
		-- output is a xor of text and keystream
		local encoded = bit.bxor(wbyte,mbyte)
		if encoded >= 256 then
			encoded = 255
		elseif encoded < 0 then
			encoded = 0
		end
		output = output..string.char(encoded)
	end
	return output;
end

-- Encrypt a value
function cryptoLib:encrypt(key,text)
	local iv = generate_iv()
	local skey = key..iv
	local encrypted = ciphersaber2(skey,text)
	return iv..encrypted
end

-- Decrypt a value
function cryptoLib:decrypt(key,cipher)
	local iv = string.sub(cipher,1,10)
	local skey = key..iv
	return ciphersaber2(skey,string.sub(cipher,11))
end

-----------------------
-- Load this library --
-----------------------

AceLibrary:Register(cryptoLib,vmajor,vminor,activate)
cryptoLib = nil